@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2007 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -40,7 +40,9 @@
 #include <kpopupmenu.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
+#include <kprogress.h>
 #include <kiconloader.h>
+#include <krun.h>
 
 #include <kbibtex_part.h>
 #include <documentlistviewitem.h>
@@ -51,6 +53,8 @@
 #include <entry.h>
 #include <macro.h>
 #include <comment.h>
+#include <preamblewidget.h>
+#include <preamble.h>
 #include <entrywidget.h>
 #include <commentwidget.h>
 #include <macrowidget.h>
@@ -62,19 +66,19 @@
 namespace KBibTeX
 {
     DocumentListView::DocumentListView( KBibTeX::DocumentWidget *docWidget, bool isReadOnly, QWidget *parent, const char *name )
-            : KListView( parent, name ), m_docWidget( docWidget ), m_contextMenu( NULL ), m_headerMenu( NULL ), m_isReadOnly( isReadOnly ), m_newElementCounter( 1 )
+            : KListView( parent, name ), m_docWidget( docWidget ), m_bibtexFile( NULL ), m_contextMenu( NULL ), m_headerMenu( NULL ), m_isReadOnly( isReadOnly ), m_newElementCounter( 1 )
     {
         setAllColumnsShowFocus( true );
         setShowSortIndicator( true );
         setSelectionMode( QListView::Extended );
-        header() ->setClickEnabled ( TRUE );
+        header() ->setClickEnabled( TRUE );
         header() ->setMovingEnabled( TRUE );
         buildColumns();
 
         //         setDragEnabled ( true );
         //         setDragAutoScroll( true );
         setAcceptDrops( TRUE );
-        setDropVisualizer ( TRUE );
+        setDropVisualizer( TRUE );
 
         connect( header(), SIGNAL( clicked( int ) ), this, SLOT( setSortingColumn( int ) ) );
         connect( this, SIGNAL( contextMenu( KListView *, QListViewItem *, const QPoint & ) ), this, SLOT( showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & ) ) );
@@ -108,31 +112,46 @@ namespace KBibTeX
     void DocumentListView::setItems()
     {
         QApplication::setOverrideCursor( Qt::waitCursor );
-        setUpdatesEnabled( FALSE );
+
+        KProgressDialog *prgDlg = new KProgressDialog( this, "prgDlg", i18n( "Updating" ), i18n( "Updating main view" ), TRUE );
+        prgDlg->show();
+        KProgress *progress = prgDlg->progressBar();
+        progress->setTotalSteps( m_bibtexFile->count() );
+
+        bool update = viewport()->isUpdatesEnabled();
+        viewport()->setUpdatesEnabled( FALSE );
 
         clear();
         for ( unsigned int i = 0; i < m_bibtexFile->count(); i++ )
         {
             BibTeX::Element *element = m_bibtexFile->at( i );
-            QListViewItem *item = new DocumentListViewItem( element, this );
+            QListViewItem *item = new DocumentListViewItem( m_bibtexFile, element, this );
             item->setVisible( m_filter.isEmpty() || element->containsPattern( m_filter, m_filterFieldType ) );
-        }
-        //     sort();
+            progress->setProgress( i );
 
-        setUpdatesEnabled( TRUE );
+            if ( i % 43 == 23 )
+                kapp->processEvents();
+        }
+
+        //setUpdatesEnabled( TRUE );
+        viewport()->setUpdatesEnabled( update );
         triggerUpdate();
+        delete prgDlg;
+
         QApplication::restoreOverrideCursor();
-        //     triggerUpdate();
     }
 
     bool DocumentListView::insertItems( BibTeX::File *items, KBibTeX::DocumentListViewItem *after )
     {
+        if ( m_bibtexFile == NULL )
+            m_bibtexFile = new BibTeX::File();
+
         for ( BibTeX::File::ElementList::iterator it = items->begin(); it != items->end(); it++ )
         {
             BibTeX::Element *element = BibTeX::File::cloneElement( *it );
 
             m_bibtexFile->appendElement( element, after == NULL ? NULL : after->element() );
-            after = new DocumentListViewItem( element, this, after );
+            after = new DocumentListViewItem( m_bibtexFile, element, this, after );
             after->setUnreadStatus( TRUE );
             m_unreadItems.append( after );
         }
@@ -144,7 +163,7 @@ namespace KBibTeX
 
     void DocumentListView::updateVisiblity()
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
 
         QListViewItemIterator it( this );
         while ( it.current() )
@@ -177,7 +196,7 @@ namespace KBibTeX
 
     void DocumentListView::restoreState()
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
         if ( settings->editing_UseSpecialFont )
             setFont( settings->editing_SpecialFont );
         else
@@ -198,7 +217,7 @@ namespace KBibTeX
             m_headerMenu->setCheckable( TRUE );
             connect( m_headerMenu, SIGNAL( activated( int ) ), this, SLOT( activateShowColumnMenu( int ) ) );
 
-            Settings * settings = Settings::self();
+            Settings * settings = Settings::self( m_bibtexFile );
 
             int item = m_headerMenu->insertItem( i18n( "Element Type" ), 0 );
             m_headerMenu->setItemChecked( item, settings->editing_MainListColumnsWidth[ 0 ] > 0 );
@@ -206,7 +225,7 @@ namespace KBibTeX
 
             for ( int i = 0; i <= ( int ) BibTeX::EntryField::ftYear - ( int ) BibTeX::EntryField::ftAbstract; i++ )
             {
-                BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType ) ( i + ( int ) BibTeX::EntryField::ftAbstract );
+                BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType )( i + ( int ) BibTeX::EntryField::ftAbstract );
                 QString label = Settings::fieldTypeToI18NString( fieldType );
                 item = m_headerMenu->insertItem( label, ( int ) fieldType + 2 );
                 m_headerMenu->setItemChecked( item, settings->editing_MainListColumnsWidth[ i + 2 ] > 0 );
@@ -221,10 +240,13 @@ namespace KBibTeX
         while ( it.current() )
         {
             DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            m_bibtexFile->deleteElement( kblvi->element() );
-            it++;
-            takeItem( kblvi );
-            delete ( kblvi );
+            if ( kblvi->isVisible() )
+            {
+                m_bibtexFile->deleteElement( kblvi->element() );
+                it++;
+                takeItem( kblvi );
+                delete( kblvi );
+            }
         }
 
         if ( above )
@@ -246,7 +268,7 @@ namespace KBibTeX
         {
             DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
             BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( kblvi->element() );
-            if ( entry != NULL )
+            if ( entry != NULL && kblvi->isVisible() )
             {
                 if ( !refs.isEmpty() )
                     refs.append( "," );
@@ -269,41 +291,61 @@ namespace KBibTeX
         QListViewItem * lvi = selectedItem();
         if ( lvi == NULL )
             lvi = currentItem();
-
         DocumentListViewItem * dlvi = NULL;
         if ( lvi != NULL )
             dlvi = dynamic_cast<KBibTeX::DocumentListViewItem *>( lvi );
 
-        BibTeX::FileImporter *importer = NULL;
-        if ( BibTeX::FileImporterBibTeX::guessCanDecode( kapp->clipboard() ->text() ) )
-            importer = new BibTeX::FileImporterBibTeX();
-
-        if ( ! importer )
+        QString clipboardText = kapp->clipboard() ->text();
+        /** check if clipboard contains BibTeX content */
+        if ( BibTeX::FileImporterBibTeX::guessCanDecode( clipboardText ) )
         {
-            // Decoding the paste text as bibtex failed. Maybe the user wants to paste the text as
-            // link address, abstract, etc...
-            if ( ! dlvi )
+            Settings * settings = Settings::self( m_bibtexFile );
+            BibTeX::FileImporter *importer = new BibTeX::FileImporterBibTeX( settings->editing_FirstNameFirst );
+            BibTeX::File *clipboardData = importer->load( clipboardText );
+            delete importer;
+
+            if ( clipboardData != NULL && clipboardData->count() > 0 )
+            {
+                bool result = insertItems( clipboardData, dlvi );
+                delete clipboardData;
+                return result;
+            }
+            else
                 return FALSE;
+        }
+        else
+        {
+            /** Decoding the paste text as bibtex failed. Maybe the user wants
+                to paste the text as link address, abstract, etc... */
+            if ( !dlvi ) // no list view item selected to add data to
+                return FALSE;
+
+            // fetch BibTeX element from current list view item
             BibTeX::Entry * element = dynamic_cast<BibTeX::Entry*>( dlvi->element() );
             if ( ! element )
                 return FALSE;
+
+            // build popup menu
             KPopupMenu * popup = new KPopupMenu( this, "pastePopup" );
             popup->insertTitle( i18n( "Paste text as..." ) );
-            for ( int i = 0; i <= ( int ) BibTeX::EntryField::ftYear; i++ )
+            for ( int i = ( int ) BibTeX::EntryField::ftAuthor; i <= ( int ) BibTeX::EntryField::ftYear; i++ )
             {
-                BibTeX::EntryField::FieldType ft;
-                ft = ( BibTeX::EntryField::FieldType ) i;
+                BibTeX::EntryField::FieldType ft = ( BibTeX::EntryField::FieldType ) i;
                 popup->insertItem( Settings::fieldTypeToI18NString( ft ), i );
             }
             popup->insertSeparator();
             QIconSet cancelPixmap = KGlobal::iconLoader() ->loadIconSet( "cancel", KIcon::Small );
             int cancelId = popup->insertItem( cancelPixmap, i18n( "Cancel" ) );
+
+            // show popup menu
             int selectedId = popup->exec( QCursor::pos() );
             if ( selectedId == cancelId || selectedId == -1 )
-                return FALSE;
+                return FALSE; // cancel menu
+
+            // determine field to add clipboard value to
             BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType ) selectedId;
             BibTeX::EntryField * field = element->getField( fieldType );
-            if ( ! field )
+            if ( field == NULL )
             {
                 field = new BibTeX::EntryField( fieldType );
                 element->addField( field );
@@ -311,36 +353,58 @@ namespace KBibTeX
             else if ( field->value() != NULL )
                 delete field->value();
 
-            BibTeX::Value * value = NULL;
+            clipboardText = BibTeX::EncoderLaTeX::currentEncoderLaTeX() ->encode( clipboardText );
+
+            // create new value from clipboard's content
+            BibTeX::Value * value = new BibTeX::Value();
             if ( fieldType == BibTeX::EntryField::ftAuthor || fieldType == BibTeX::EntryField::ftEditor )
-                value = new BibTeX::ValuePersons();
+            {
+                Settings * settings = Settings::self( m_bibtexFile );
+                value->items.append( new BibTeX::PersonContainer( clipboardText, settings->editing_FirstNameFirst ) );
+            }
+            else if ( fieldType == BibTeX::EntryField::ftKeywords )
+                value->items.append( new BibTeX::KeywordContainer( clipboardText ) );
             else
-                value = new BibTeX::Value();
-            QString stringValue = kapp->clipboard() ->text();
-            // FIXME: I assume that the encoding is correct here; i am not sure if
-            // this is ok; maybe I need a construct like in
-            // BibTeXFileImporterBibTeX::load() here;
-            // maybe Thomas Fischer knows if this needs fixing...
-            // TF: I assume that all clipboard content is UTF-8 ...
-            stringValue = BibTeX::EncoderLaTeX::currentEncoderLaTeX() ->encode( stringValue );
+                value->items.append( new BibTeX::PlainText( clipboardText ) );
 
-            value->add( new BibTeX::ValueItem( stringValue, FALSE ) );
             field->setValue( value );
+
             return TRUE;
         }
+    }
 
-        BibTeX::File *clipboardData = importer->load( kapp->clipboard() ->text() );
-        delete importer;
+    void DocumentListView::sendSelectedToLyx()
+    {
+        QStringList refsToSend;
+        QListViewItemIterator it( this, QListViewItemIterator::Selected );
+        while ( it.current() )
+        {
+            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
+            BibTeX::Entry *entry = dynamic_cast<BibTeX::Entry*>( kblvi->element() );
+            if ( entry != NULL && kblvi->isVisible() )
+                refsToSend.append( entry->id() );
+            it++;
+        }
 
-        if ( clipboardData != NULL )
-            return insertItems( clipboardData, dlvi );
+        Settings * settings = Settings::self( m_bibtexFile );
+        QString lyxPipeFilename = settings->detectLyXInPipe();
+        kdDebug() << "sendSelectedToLyx: lyxPipeFilename= " << lyxPipeFilename << endl;
+        QFile pipe( lyxPipeFilename );
+        if ( pipe.exists() && pipe.open( IO_WriteOnly ) )
+        {
+            QTextStream * writer = new QTextStream( &pipe );
+            QString msg = "LYXCMD:kbibtex:citation-insert:" + refsToSend.join( "," );
+            *writer << msg << endl;
+            delete writer;
+            pipe.close();
+        }
         else
-            return FALSE;
+            KMessageBox::error( this, ( lyxPipeFilename.isEmpty() ? i18n( "Cannot establish a link to LyX" ) : QString( i18n( "Cannot establish a link to LyX using the pipe \"%1\"" ) ).arg( lyxPipeFilename ) ) + i18n( "\nMaybe LyX is not running?" ), i18n( "Error communicating with LyX" ) );
     }
 
     void DocumentListView::slotDoubleClick( QListViewItem *item )
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
         bool openingDocumentOK = FALSE;
 
         if ( settings->editing_MainListDoubleClickAction == 1 )
@@ -363,7 +427,7 @@ namespace KBibTeX
                         KURL url = KURL( *it );
                         if ( url.isValid() && ( !url.isLocalFile() || QFile::exists( url.path() ) ) )
                         {
-                            kapp->invokeBrowser( url.prettyURL() );
+                            new KRun( url, this );
                             openingDocumentOK = TRUE;
                             break;
                         }
@@ -373,7 +437,7 @@ namespace KBibTeX
         }
 
         if ( !openingDocumentOK )
-            editElement( item );
+            m_docWidget->editElement( );
     }
 
 
@@ -412,9 +476,21 @@ namespace KBibTeX
                     {
                         BibTeX::Macro* macro = dynamic_cast<BibTeX::Macro*>( lvi->element() );
                         if ( macro )
+                        {
                             dialogResult = KBibTeX::MacroWidget::execute( macro, m_isReadOnly );
-                        if ( dialogResult == QDialog::Accepted )
-                            result = macro;
+                            if ( dialogResult == QDialog::Accepted )
+                                result = macro;
+                        }
+                        else
+                        {
+                            BibTeX::Preamble* preamble = dynamic_cast<BibTeX::Preamble*>( lvi->element() );
+                            if ( preamble )
+                            {
+                                dialogResult = KBibTeX::PreambleWidget::execute( preamble, m_isReadOnly );
+                                if ( dialogResult == QDialog::Accepted )
+                                    result = preamble;
+                            }
+                        }
                     }
                 }
 
@@ -467,32 +543,28 @@ namespace KBibTeX
 
     void DocumentListView::setSortingColumn( int column )
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
         settings->editing_MainListSortingColumn = column;
         settings->editing_MainListSortingOrder = ( sortOrder() == Qt::Ascending ) ? 1 : -1;
     }
 
     bool DocumentListView::acceptDrag( QDropEvent * event ) const
     {
-        return QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );;
+        return QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );
     }
 
-    void DocumentListView::saveColumnIndex( int col )
+    void DocumentListView::saveColumnIndex()
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
         QHeader *hdr = header();
 
-        int from = col == -1 ? 0 : col, to = col == -1 ? columns() : ( col + 1 );
-
-        for ( int i = from; i < to; i++ )
-        {
+        for ( int i = 0; i < columns(); i++ )
             settings->editing_MainListColumnsIndex[ i ] = hdr->mapToIndex( i );
-        }
     }
 
     void DocumentListView::restoreColumnIndex()
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
         QHeader *hdr = header();
 
         for ( int i = 0; i < columns(); i++ )
@@ -501,7 +573,7 @@ namespace KBibTeX
 
     void DocumentListView::saveColumnWidths( int col )
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
 
         int from = col == -1 ? 0 : col, to = col == -1 ? columns() : ( col + 1 );
 
@@ -516,7 +588,7 @@ namespace KBibTeX
 
     void DocumentListView::restoreColumnWidths()
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
 
         for ( int col = 0; col < columns(); col++ )
         {
@@ -527,7 +599,7 @@ namespace KBibTeX
 
     void DocumentListView::restoreSortingColumn()
     {
-        Settings * settings = Settings::self();
+        Settings * settings = Settings::self( m_bibtexFile );
         setSortColumn( settings->editing_MainListSortingColumn );
         setSortOrder( settings->editing_MainListSortingOrder > 0 ? Qt::Ascending : Qt::Descending );
     }
@@ -548,9 +620,12 @@ namespace KBibTeX
         QString text;
         QStrList urlList;
 
-        if ( QUriDrag::decode( event, urlList ) )
+        if ( QTextDrag::decode( event, text ) && KURL( text ).isValid() )
+            urlList.append( text );
+
+        if ( !urlList.isEmpty() || QUriDrag::decode( event, urlList ) )
         {
-            QString url = urlList.at ( 0 );
+            QString url = urlList.at( 0 );
             QString tmpFile;
             if ( ! KIO::NetAccess::download( url, tmpFile, 0 ) )
             {
@@ -568,16 +643,18 @@ namespace KBibTeX
             text = QString( ba );
             f.close();
             KIO::NetAccess::removeTempFile( tmpFile );
-
         }
-        else if ( ! QTextDrag::decode( event, text ) )
-            return ;
+        else if ( !QTextDrag::decode( event, text ) )
+            return;
 
         event->accept( TRUE );
         DocumentListViewItem * dlvi = dynamic_cast<KBibTeX::DocumentListViewItem *>( item );
         BibTeX::FileImporter *importer = NULL;
         if ( BibTeX::FileImporterBibTeX::guessCanDecode( text ) )
-            importer = new BibTeX::FileImporterBibTeX();
+        {
+            Settings * settings = Settings::self( m_bibtexFile );
+            importer = new BibTeX::FileImporterBibTeX( settings->editing_FirstNameFirst );
+        }
         //        else if (BibTeX::FileImporterExternal::guessCanDecode( text ))
         //            importer = new BibTeX::FileImporterExternal();
         else
@@ -621,7 +698,8 @@ namespace KBibTeX
         while ( it.current() )
         {
             DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
-            exporter->save( &buffer, kblvi->element() );
+            if ( kblvi->isVisible() )
+                exporter->save( &buffer, kblvi->element() );
             it++;
         }
         delete exporter;
@@ -650,19 +728,18 @@ namespace KBibTeX
                     break;
                 }
 
-            default: break;
+            default:
+                break;
             }
         }
 
         return KListView::eventFilter( watched, e );
     }
 
-    void DocumentListView::keyPressEvent ( QKeyEvent *e )
+    void DocumentListView::keyPressEvent( QKeyEvent *e )
     {
         if ( e->key() == QKeyEvent::Key_Enter || e->key() == QKeyEvent::Key_Return )
-        {
             slotDoubleClick();
-        }
         else
             KListView::keyPressEvent( e );
     }
@@ -698,7 +775,7 @@ namespace KBibTeX
 
         for ( int i = 0; i <= ( int ) BibTeX::EntryField::ftYear - ( int ) BibTeX::EntryField::ftAbstract; i++ )
         {
-            BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType ) ( i + ( int ) BibTeX::EntryField::ftAbstract );
+            BibTeX::EntryField::FieldType fieldType = ( BibTeX::EntryField::FieldType )( i + ( int ) BibTeX::EntryField::ftAbstract );
             QString label = Settings::fieldTypeToI18NString( fieldType );
             addColumn( label );
         }
