@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2007 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -24,6 +24,7 @@
 #include <qtimer.h>
 #include <qtooltip.h>
 #include <qpushbutton.h>
+#include <qcheckbox.h>
 
 #include <klistview.h>
 #include <klineedit.h>
@@ -41,7 +42,7 @@ namespace KBibTeX
 {
 
     FieldListView::FieldListView( const QString& caption, bool isReadOnly, QWidget *parent, const char *name )
-            : QWidget( parent, name ), m_value( new BibTeX::Value() ), m_caption( caption ), m_isReadOnly( isReadOnly ), m_enabled( TRUE ), m_isModified( FALSE ), m_newValueCounter( 0 ), m_fieldType( BibTeX::EntryField::ftUnknown )
+            : QWidget( parent, name ), m_value( new BibTeX::Value() ), m_caption( caption ), m_isReadOnly( isReadOnly ), m_enabled( TRUE ), m_isComplex( FALSE ), m_isModified( FALSE ), m_newValueCounter( 0 ), m_fieldType( BibTeX::EntryField::ftUnknown )
     {
         setupGUI();
         m_listViewElements->installEventFilter( this );
@@ -64,22 +65,11 @@ namespace KBibTeX
                 delete m_value;
 
             if ( value != NULL )
-            {
-                const BibTeX::ValuePersons * personsValue = dynamic_cast<const BibTeX::ValuePersons*>( value );
-                if ( personsValue != NULL )
-                    m_value = new BibTeX::ValuePersons( personsValue );
-                else
-                    m_value = new BibTeX::Value( value );
-            }
+                m_value = new BibTeX::Value( value );
             else
-            {
-                if ( m_fieldType == BibTeX::EntryField::ftAuthor || m_fieldType == BibTeX::EntryField::ftEditor )
-                    m_value = new BibTeX::ValuePersons( );
-                else
-                    m_value = new BibTeX::Value( );
-            }
+                m_value = new BibTeX::Value( );
 
-            updateListView();
+            reset();
             updateGUI();
 
             m_isModified = FALSE;
@@ -88,16 +78,10 @@ namespace KBibTeX
 
     BibTeX::Value *FieldListView::value()
     {
-        if ( m_value->isEmpty() )
+        if ( m_value->items.isEmpty() )
             return NULL;
         else
-        {
-            const BibTeX::ValuePersons * personsValue = dynamic_cast<const BibTeX::ValuePersons*>( m_value );
-            if ( personsValue != NULL )
-                return new BibTeX::ValuePersons( personsValue );
-            else
-                return new BibTeX::Value( m_value );
-        }
+            return new BibTeX::Value( m_value );
     }
 
     void FieldListView::setEnabled( bool enabled )
@@ -113,11 +97,7 @@ namespace KBibTeX
         Settings * settings = Settings::self();
         m_listViewElements->renameLineEdit() ->setCompletionObject( settings->completion( m_fieldType ) );
 
-        delete m_value;
-        if ( m_fieldType == BibTeX::EntryField::ftAuthor || m_fieldType == BibTeX::EntryField::ftEditor )
-            m_value = new BibTeX::ValuePersons( );
-        else
-            m_value = new BibTeX::Value( );
+        m_value->items.clear();
     }
 
     QString FieldListView::caption()
@@ -127,7 +107,7 @@ namespace KBibTeX
 
     bool FieldListView::isEmpty()
     {
-        return m_value != NULL ? m_value->isEmpty() : TRUE;
+        return m_value != NULL ? m_value->items.isEmpty() : TRUE;
     }
 
     bool FieldListView::isModified()
@@ -135,7 +115,7 @@ namespace KBibTeX
         return m_isModified;
     }
 
-    bool FieldListView::eventFilter ( QObject *o, QEvent * e )
+    bool FieldListView::eventFilter( QObject *o, QEvent * e )
     {
         if ( o == m_listViewElements->renameLineEdit() )
         {
@@ -148,42 +128,36 @@ namespace KBibTeX
             //override delete action
             if ( ke->key() == Key_Delete && ke->state() == NoButton )
             {
-                kdDebug() << "Key_Delete" << endl;
                 slotDelete();
                 ke->accept();
                 return true;
             }
             else if ( ke->key() == Key_F2 && ke->state() == NoButton )
             {
-                kdDebug() << "Key_F2" << endl;
                 slotEdit();
                 ke->accept();
                 return true;
             }
             else if ( ke->key() == Key_A && ke->state() == ControlButton )
             {
-                kdDebug() << "Ctrl + Key_A" << endl;
                 slotAdd();
                 ke->accept();
                 return true;
             }
             else if ( ke->key() == Key_Up && ke->state() == ControlButton )
             {
-                kdDebug() << "Ctrl + Key_Up" << endl;
                 slotUp();
                 ke->accept();
                 return true;
             }
             else if ( ke->key() == Key_Down && ke->state() == ControlButton )
             {
-                kdDebug() << "Ctrl + Key_Down" << endl;
                 slotDown();
                 ke->accept();
                 return true;
             }
             else if ( ke->key() == Key_C && ke->state() == ControlButton | AltButton )
             {
-                kdDebug() << "Ctrl + Alt + Key_C" << endl;
                 slotComplex();
                 ke->accept();
                 return true;
@@ -195,7 +169,8 @@ namespace KBibTeX
 
     void FieldListView::updateGUI()
     {
-        if ( m_value != NULL && isSimple() )
+        disconnect( m_checkBoxEtAl, SIGNAL( toggled( bool ) ), this, SLOT( apply() ) );
+        if ( m_value != NULL && !m_isComplex )
         {
             bool isElementSelected = m_listViewElements->selectedItem() != NULL;
             m_pushButtonAdd->setEnabled( !m_isReadOnly );
@@ -204,6 +179,7 @@ namespace KBibTeX
             m_pushButtonUp->setEnabled( !m_isReadOnly && isElementSelected && m_listViewElements->selectedItem() != m_listViewElements->firstChild() );
             m_pushButtonDown->setEnabled( !m_isReadOnly && isElementSelected && m_listViewElements->selectedItem() != m_listViewElements->lastItem() );
             m_listViewElements->setEnabled( !m_isReadOnly );
+            m_checkBoxEtAl->setEnabled( !m_isReadOnly );
         }
         else
         {
@@ -213,7 +189,9 @@ namespace KBibTeX
             m_pushButtonUp->setEnabled( FALSE );
             m_pushButtonDown->setEnabled( FALSE );
             m_listViewElements->setEnabled( FALSE );
+            m_checkBoxEtAl->setEnabled( FALSE );
         }
+        connect( m_checkBoxEtAl, SIGNAL( toggled( bool ) ), this, SLOT( apply() ) );
     }
 
     void FieldListView::slotAdd()
@@ -243,7 +221,7 @@ namespace KBibTeX
         if ( isSimple() && item != NULL )
         {
             delete item;
-            updateValue();
+            apply();
             updateGUI();
 
             m_isModified = TRUE;
@@ -256,7 +234,7 @@ namespace KBibTeX
         if ( isSimple() && !m_listViewElements->isRenaming() && item != NULL && item -> itemAbove() != NULL )
         {
             item->itemAbove() ->moveItem( item );
-            updateValue();
+            apply();
             updateGUI();
 
             m_isModified = TRUE;
@@ -269,7 +247,7 @@ namespace KBibTeX
         if ( isSimple() && !m_listViewElements->isRenaming() && item != NULL && item -> itemBelow() != NULL )
         {
             item->moveItem( item->itemBelow() );
-            updateValue();
+            apply();
             updateGUI();
 
             m_isModified = TRUE;
@@ -278,9 +256,9 @@ namespace KBibTeX
 
     void FieldListView::slotComplex()
     {
-        if ( !m_listViewElements->isRenaming() && ValueWidget::execute( m_caption, m_value, m_isReadOnly, this ) == QDialog::Accepted )
+        if ( !m_listViewElements->isRenaming() && ValueWidget::execute( m_caption, m_fieldType, m_value, m_isReadOnly, this ) == QDialog::Accepted )
         {
-            updateListView();
+            reset();
             updateGUI();
 
             m_isModified = TRUE;
@@ -295,14 +273,13 @@ namespace KBibTeX
 
     void FieldListView::slotItemRenamed( QListViewItem * item, int /*col*/, const QString & text )
     {
-        kdDebug( ) << "FieldListView::slotItemRenamed" << endl;
         if ( text.isEmpty() && isSimple() && item != NULL )
         {
             delete item;
             updateGUI();
         }
 
-        updateValue();
+        apply();
         m_isModified = TRUE;
     }
 
@@ -310,7 +287,7 @@ namespace KBibTeX
     {
         Settings * settings = Settings::self();
 
-        QGridLayout * layout = new QGridLayout( this, 7, 2, 0, KDialog::marginHint() );
+        QGridLayout * layout = new QGridLayout( this, 8, 2, 0, KDialog::marginHint() );
         layout->setRowStretch( 5, 1 );
 
         m_listViewElements = new KListView( this );
@@ -369,60 +346,104 @@ namespace KBibTeX
         m_pushButtonComplexEdit->setIconSet( QIconSet( SmallIcon( "leftjust" ) ) );
         connect( m_pushButtonComplexEdit, SIGNAL( clicked() ), this, SLOT( slotComplex() ) );
         QToolTip::add( m_pushButtonComplexEdit, QString( i18n( "Edit current ”%1“ item as a concatenated string (Ctrl+Alt+C)" ) ).arg( m_caption ) );
+
+        m_checkBoxEtAl = new QCheckBox( i18n( "... and others (et al.)" ), this );
+        layout->addMultiCellWidget( m_checkBoxEtAl, 7, 7, 0, 1 );
+        connect( m_checkBoxEtAl, SIGNAL( toggled( bool ) ), this, SLOT( apply() ) );
     }
 
-    void FieldListView::updateValue()
+    void FieldListView::apply()
     {
-        QString text;
+        QStringList elements;
+        Settings * settings = Settings::self();
+        KCompletion *completion = settings->completion( m_fieldType );
+
+        m_value->items.clear();
 
         for ( QListViewItemIterator it( m_listViewElements ); it.current(); it++ )
         {
-            if ( !text.isEmpty() )
-                text.append( " and " );
-            text.append( it.current() ->text( 0 ) );
+            QString text = it.current() ->text( 0 );
+            elements.append( text );
+            completion->addItem( text );
         }
 
-        m_value->clear();
-        if ( !text.isEmpty() )
+        if ( elements.count() > 0 )
         {
-            BibTeX::ValueItem * item = new BibTeX::ValueItem ( text, FALSE );
-            m_value->add( item );
-            Settings * settings = Settings::self();
-            KCompletion *completion = settings->completion( m_fieldType );
+            m_value->items.clear();
+            BibTeX::PersonContainer *container = new BibTeX::PersonContainer( settings->editing_FirstNameFirst );
 
-            if ( m_fieldType == BibTeX::EntryField::ftAuthor || m_fieldType == BibTeX::EntryField::ftEditor )
+            switch ( m_fieldType )
             {
-                QStringList elements = QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", FALSE ), text );
-                for ( QStringList::Iterator it = elements.begin(); it != elements.end(); it++ )
-                    completion ->addItem( *it );
+            case BibTeX::EntryField::ftAuthor:
+            case BibTeX::EntryField::ftEditor:
+                {
+                    for ( QStringList::ConstIterator it = elements.constBegin(); it != elements.constEnd(); ++it )
+                        container->persons.append( new BibTeX::Person( *it, settings->editing_FirstNameFirst ) );
+                }
+                break;
+            default:
+                kdDebug() << "Don't know how to handle entries of type " << BibTeX::EntryField::fieldTypeToString( m_fieldType ) << endl;
             }
+
+            if ( m_checkBoxEtAl->isChecked() )
+                container->persons.append( new BibTeX::Person( "others", settings->editing_FirstNameFirst ) );
+
+            if ( !container->persons.isEmpty() )
+                m_value->items.append( container );
             else
-                completion->addItem( text );
+                delete container;
         }
     }
 
-    void FieldListView::updateListView()
+    void FieldListView::reset()
     {
+        disconnect( m_checkBoxEtAl, SIGNAL( toggled( bool ) ), this, SLOT( apply() ) );
         m_listViewElements->clear();
-        if ( m_value->count() == 1 && !m_value->first() ->isStringKey() )
+        m_checkBoxEtAl->setChecked( FALSE );
+
+        m_isComplex = FALSE;
+        for ( QValueList<BibTeX::ValueItem*>::ConstIterator it = m_value->items.constBegin(); !m_isComplex && it != m_value->items.constEnd(); ++it )
         {
-            QStringList elements = QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", FALSE ), m_value->first() ->text() );
-            for ( QStringList::Iterator it = elements.begin(); it != elements.end(); it++ )
-                new QListViewItem( m_listViewElements, m_listViewElements->lastItem(), *it );
+            BibTeX::MacroKey *macroKey = dynamic_cast<BibTeX::MacroKey *>( *it );
+            m_isComplex = macroKey != NULL;
         }
 
+        if ( !m_isComplex )
+            switch ( m_fieldType )
+            {
+            case BibTeX::EntryField::ftAuthor:
+            case BibTeX::EntryField::ftEditor:
+                {
+                    for ( QValueList<BibTeX::ValueItem*>::ConstIterator it = m_value->items.constBegin(); it != m_value->items.constEnd(); ++it )
+                    {
+                        BibTeX::PersonContainer *container = dynamic_cast<BibTeX::PersonContainer *>( *it );
+                        if ( container != NULL )
+                            for ( QValueList<BibTeX::Person*>::ConstIterator pit = container->persons.constBegin(); pit != container->persons.constEnd(); ++pit )
+                            {
+                                QString text = ( *pit )->text();
+                                if ( text == "others" )
+                                    m_checkBoxEtAl->setChecked( TRUE );
+                                else
+                                    new QListViewItem( m_listViewElements, m_listViewElements->lastItem(), text );
+                            }
+                    }
+                }
+                break;
+            default:
+                kdDebug() << "Don't know how to handle entries of type " << BibTeX::EntryField::fieldTypeToString( m_fieldType ) << endl;
+            }
+
+        connect( m_checkBoxEtAl, SIGNAL( toggled( bool ) ), this, SLOT( apply() ) );
     }
 
     bool FieldListView::isSimple()
     {
-        return m_value->count() == 0 || ( m_value->count() == 1 && !m_value->first() ->isStringKey() );
+        return m_value->items.count() == 0 || ( m_value->items.count() == 1 && dynamic_cast<BibTeX::MacroKey*>( m_value->items.first() ) == NULL );
     }
 
     void FieldListView::itemRenameDone()
     {
-        //         KListViewItem * item = static_cast<KListViewItem*>( m_listViewElements->selectedItem() );
-        kdDebug() << "itemRenameDone" << endl;
-        updateValue();
+        apply();
     }
 }
 #include "fieldlistview.moc"
