@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2007 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -18,6 +18,7 @@
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 #include <string>
+#include <fcntl.h>
 
 #include <qstring.h>
 #include <qsplitter.h>
@@ -51,7 +52,10 @@
 #include <kio/netaccess.h>
 #include <kmainwindow.h>
 
-#include <comment.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <entry.h>
 #include <documentlistviewitem.h>
@@ -60,17 +64,24 @@
 #include <fileexporterbibtex.h>
 #include <fileexporterxml.h>
 #include <fileexporterxslt.h>
-#include <fileimporter.h>
-#include <fileimporterbibtex.h>
 #include <webquery.h>
 #include <webquerypubmed.h>
+#include <webquerybibsonomy.h>
+// #include <webquerygooglescholar.h>
+#include <webqueryamatex.h>
+#include <webqueryspireshep.h>
+#include <webquerycitebase.h>
+#include <webquerydblp.h>
+#include <webqueryzmath.h>
 #include <settings.h>
 #include <encoderlatex.h>
 #include "kbibtex_part.h"
 
+const QString inPipeFilename = QDir::homeDirPath() + "/.kbibtex.in";
+
 KBibTeXPart::KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                           QObject *parent, const char *name )
-        : KParts::ReadWritePart( parent, name ), m_defInitCounter( 0 ), m_settingsDlg( NULL ), m_initializationDone( FALSE )
+        : KParts::ReadWritePart( parent, name ), m_defInitCounter( 0 ), m_settingsDlg( NULL ), m_initializationDone( FALSE ), m_inPipe( NULL )
 {
     m_mainWindow = dynamic_cast<KMainWindow*>( parent );
     if ( m_mainWindow == NULL )
@@ -142,8 +153,6 @@ void KBibTeXPart::setModified( bool modified )
 
 bool KBibTeXPart::queryClose()
 {
-    kdDebug() << "KBibTeXPart::queryClose" << endl;
-
     writeSettings();
 
     if ( !isReadWrite() || !isModified() )
@@ -167,13 +176,9 @@ bool KBibTeXPart::queryClose()
         if ( !handled )
         {
             if ( m_url.isEmpty() )
-            {
                 return saveAs();
-            }
             else
-            {
                 save();
-            }
         }
         else if ( abortClose ) return FALSE;
         return waitSaveComplete();
@@ -187,8 +192,6 @@ bool KBibTeXPart::queryClose()
 
 bool KBibTeXPart::closeURL()
 {
-    kdDebug() << "KBibTeXPart::closeURL" << endl;
-
     writeSettings();
 
     if ( isReadWrite() && isModified() )
@@ -229,7 +232,11 @@ bool KBibTeXPart::saveAs()
     bool result = FALSE;
 
     QString startDir = ! url().isEmpty() ? url().url() : QDir::currentDirPath();
-    KURL saveURL = KFileDialog::getSaveURL( startDir, "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );
+    KURL saveURL = KFileDialog::getSaveURL( startDir, "*.bib|" + i18n( "BibTeX (*.bib)" ) + "\n*.ris|" + i18n( "Reference Manager (*.ris)" )
+#ifdef HAVE_LIBBIBUTILS
+                                            + "\n*.enl|" + i18n( "EndNote (text only) (*.enl)" )
+#endif
+                                            , widget() );
 
     if ( saveURL.isValid() && !saveURL.isEmpty() )
     {
@@ -250,7 +257,11 @@ bool KBibTeXPart::saveAs()
 void KBibTeXPart::slotFileExport()
 {
     QString startDir = ! url().isEmpty() ? url().url() : QDir::currentDirPath();
-    KURL exportURL = KFileDialog::getSaveURL( startDir, "*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ) + "\n*.html|" + i18n( "Hypertext Markup Langage (*.html)" ) + "\n*.rtf|" + i18n( "Rich Text Format (*.rtf)" ), widget() );
+    KURL exportURL = KFileDialog::getSaveURL( startDir, "*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.ris|" + i18n( "Reference Manager (*.ris)" )
+#ifdef HAVE_LIBBIBUTILS
+                     + "\n*.enl|" + i18n( "EndNote (text only) (*.enl)" )
+#endif
+                     + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ) + "\n*.html|" + i18n( "Hypertext Markup Langage (*.html)" ) + "\n*.rtf|" + i18n( "Rich Text Format (*.rtf)" ), widget() );
 
     if ( exportURL.isValid() && !exportURL.isEmpty() )
     {
@@ -266,6 +277,12 @@ void KBibTeXPart::slotFileExport()
             format = BibTeX::File::formatPDF;
         else if ( exportURL.fileName().endsWith( ".ps" ) )
             format = BibTeX::File::formatPS;
+#ifdef HAVE_LIBBIBUTILS
+        else if ( exportURL.fileName().endsWith( ".enl" ) )
+            format = BibTeX::File::formatEndNote;
+#endif
+        else if ( exportURL.fileName().endsWith( ".ris" ) )
+            format = BibTeX::File::formatRIS;
         else if ( exportURL.fileName().endsWith( ".xml" ) )
             format = BibTeX::File::formatXML;
         else if ( exportURL.fileName().endsWith( ".html" ) )
@@ -283,17 +300,62 @@ void KBibTeXPart::slotFileExport()
         if ( error )
         {
 #if (KDE_VERSION_MINOR>=4) && (KDE_VERSION_MAJOR>=3)
-            KMessageBox::errorList( widget(), QString( i18n( "File could not be exported in \"%1\"." ) ).arg( exportURL.prettyURL() ).append( i18n( "\n\nSee the following error log for details." ) ), errorLog, i18n( "Exporting failed" ) );
-#else
-            KMessageBox::error( widget(), QString( i18n( "File could not be exported in \"%1\"." ) ).arg( exportURL.prettyURL() ), i18n( "Exporting failed" ) );
+            if ( !errorLog.empty() )
+                KMessageBox::errorList( widget(), QString( i18n( "File could not be exported in \"%1\"." ) ).arg( exportURL.prettyURL() ).append( i18n( "\n\nSee the following error log for details." ) ), errorLog, i18n( "Exporting failed" ) );
+            else
 #endif
-
+                KMessageBox::error( widget(), QString( i18n( "File could not be exported to \"%1\"." ) ).arg( exportURL.prettyURL() ), i18n( "Exporting failed" ) );
         }
 
     }
 
 }
 
+void KBibTeXPart::slotFileStatistics()
+{
+    m_documentWidget->showStatistics();
+}
+
+void KBibTeXPart::slotFileMerge()
+{
+    QString startDir = ! url().isEmpty() ? url().url() : QDir::currentDirPath();
+    KURL mergeURL = KFileDialog::getOpenURL( startDir, QString( "*.bib *.ris" ) +
+#ifdef HAVE_LIBBIBUTILS
+                    " *.enl" +
+#endif
+                    "|" + i18n( "Supported Bibliographies" ) + "\n*.bib|" + i18n( "BibTeX (*.bib)" ) + "\n*.ris|" + i18n( "Reference Manager (*.ris)" )
+#ifdef HAVE_LIBBIBUTILS
+                    + "\n*.enl|" + i18n( "EndNote (text only) (*.enl)" )
+#endif
+                    , widget() );
+
+    if ( !mergeURL.isValid() || mergeURL.isEmpty() )
+        return;
+
+    if ( !KIO::NetAccess::exists( mergeURL, TRUE, widget() ) )
+    {
+        KMessageBox::error( widget(), i18n( "The given file could not be read, check if it exists or if it is readable for the current user." ) );
+        return ;
+    }
+
+    KTempFile tempBibFile = KTempFile( locateLocal( "tmp", "bibmerge" ) );
+    tempBibFile.setAutoDelete( TRUE );
+    bool error = !KIO::NetAccess::file_copy( mergeURL, KURL( tempBibFile.name() ),  -1, TRUE, FALSE, widget() );
+
+    if ( !error )
+        error = !m_documentWidget->open( tempBibFile.file(), TRUE );
+
+    tempBibFile.close();
+
+    if ( error )
+    {
+        KMessageBox::error( widget(), i18n( "The given file could not be merged." ) );
+        return;
+    }
+    else
+        setModified( TRUE );
+}
+
 void KBibTeXPart::slotPreferences()
 {
     if ( !m_settingsDlg )
@@ -319,8 +381,11 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
         KStdAction::save( this, SLOT( save() ), actionCollection() ) ->setToolTip( i18n( "Save the current file" ) );
         KStdAction::saveAs( this, SLOT( slotFileSaveAs() ), actionCollection() ) ->setToolTip( i18n( "Save the current file under a new filename" ) );
     }
+    m_actionFileStatistics = new KAction( i18n( "Statistics..." ), CTRL + SHIFT + Key_I, this, SLOT( slotFileStatistics() ), actionCollection(), "file_statistics" );
     m_actionFileExport = new KAction( i18n( "Export..." ), SmallIconSet( "goto" ), CTRL + Key_E, this, SLOT( slotFileExport() ), actionCollection(), "file_export" );
     m_actionFileExport->setToolTip( i18n( "Export to a different file format" ) );
+    m_actionFileMerge = new KAction( i18n( "Merge..." ), SmallIconSet( "reload" ), CTRL + Key_M, this, SLOT( slotFileMerge() ), actionCollection(), "file_merge" );
+    m_actionFileMerge->setToolTip( i18n( "Merge BibTeX entries from another file" ) );
 
     m_actionEditCut = KStdAction::cut( m_documentWidget, SLOT( cutElements() ), actionCollection() );
     m_actionEditCut->setToolTip( i18n( "Cut the selected BibTeX elements to the clipboard" ) );
@@ -332,6 +397,9 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
     m_actionEditCopyRef->setToolTip( i18n( "Copy a reference of the selected BibTeX elements to the clipboard" ) );
     m_actionEditPaste = KStdAction::paste( m_documentWidget, SLOT( pasteElements() ), actionCollection() );
     m_actionEditPaste->setToolTip( i18n( "Paste BibTeX elements from the clipboard" ) );
+    m_actionElementSendToLyX = new KAction( i18n( "Send Reference to &LyX" ), CTRL + SHIFT + Key_L, m_documentWidget, SLOT( sendSelectedToLyx() ), actionCollection(), "element_sendtolyx" );
+    m_actionElementSendToLyX->setToolTip( i18n( "Send a reference of the selected BibTeX elements to LyX" ) );
+    m_actionElementSendToLyX->setEnabled( FALSE );
     m_actionEditFind = KStdAction::find( m_documentWidget, SLOT( find() ), actionCollection() );
     m_actionEditFind->setToolTip( i18n( "Find text" ) );
     m_actionEditFindNext = KStdAction::findNext( m_documentWidget, SLOT( findNext() ), actionCollection() );
@@ -344,38 +412,63 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
     connect( m_actionMenuSearchWebsites->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotSearchWebsites( int ) ) );
     m_actionMenuSearchWebsites->setEnabled( FALSE );
 
-    m_actionMenuSearchOnlineDatabases = new KActionMenu( i18n( "Search Online Databases" ), actionCollection(), "search_onlinedatabases" );
+    m_actionMenuSearchOnlineDatabases = new KActionMenu( i18n( "Search Online Databases" ), SmallIconSet( "network" ), actionCollection(), "search_onlinedatabases" );
     m_actionMenuSearchOnlineDatabases->setToolTip( i18n( "Search online databases to import bibliography data" ) );
     connect( m_actionMenuSearchOnlineDatabases->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotSearchOnlineDatabases( int ) ) );
     KBibTeX::WebQuery *webQuery = new KBibTeX::WebQueryPubMed( parentWidget );
     webQueryList.append( webQuery );
     m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+    webQuery = new KBibTeX::WebQueryBibSonomy( parentWidget );
+    webQueryList.append( webQuery );
+    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+    webQuery = new KBibTeX::WebQuerySpiresHep( parentWidget );
+    webQueryList.append( webQuery );
+    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+    webQuery = new KBibTeX::WebQueryZMATH( parentWidget );
+    webQueryList.append( webQuery );
+    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+    webQuery = new KBibTeX::WebQueryCitebase( parentWidget );
+    webQueryList.append( webQuery );
+    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+    webQuery = new KBibTeX::WebQueryDBLP( parentWidget );
+    webQueryList.append( webQuery );
+    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+//     webQuery = new KBibTeX::WebQueryGoogleScholar( parentWidget );
+//     webQueryList.append( webQuery );
+//     m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
+    webQuery = new KBibTeX::WebQueryAmatex( parentWidget );
+    webQueryList.append( webQuery );
+    m_actionMenuSearchOnlineDatabases->popupMenu() ->insertItem( webQuery->title(), webQuery->uniqueId() );
 
     m_actionViewShowColumns = new KActionMenu( i18n( "&Show Columns" ), actionCollection(), "view_showcolumns" );
     m_actionViewShowColumns->setToolTip( i18n( "Select columns to show in the main list" ) );
     m_actionViewDocument = new KActionMenu( i18n( "View Document" ), actionCollection(), "view_document" );
     m_actionViewDocument->setToolTip( i18n( "View the documents associated with this BibTeX entry" ) );
     m_actionViewDocument->setEnabled( FALSE );
+    m_actionAssignKeywords = new KActionMenu( i18n( "Assign Keywords" ), actionCollection(), "assign_keywords" );
+    m_actionAssignKeywords->setToolTip( i18n( "Assign keywords to this entry" ) );
+    m_actionAssignKeywords->setEnabled( FALSE );
 
     m_actionPreferences = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );
     m_actionPreferences->setToolTip( i18n( "Configure KBibTeX" ) );
-    KAction *actionSourceEditor = new KAction( i18n( "Configure Editor..." ), 0, m_documentWidget, SLOT( configureEditor() ), actionCollection(), "options_configure_editor" );
-    actionSourceEditor ->setToolTip( i18n( "Configure the source editor" ) );
+    KAction *actionSourceEditor = new KAction( i18n( "Configure Source View..." ), 0, m_documentWidget, SLOT( configureEditor() ), actionCollection(), "options_configure_editor" );
+    actionSourceEditor ->setToolTip( i18n( "Configure the Source Vditor" ) );
 
-    ( new KAction( i18n( "Article" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_article" ) ) ->setToolTip( i18n( "Add an article to the BibTeX file" ) );
-    ( new KAction( i18n( "Book" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_book" ) ) ->setToolTip( i18n( "Add a book to the BibTeX file" ) );
+    ( new KAction( i18n( "Article" ), "element_new_entry_article", CTRL + Key_A, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_article" ) ) ->setToolTip( i18n( "Add an article to the BibTeX file" ) );
+    ( new KAction( i18n( "Book" ), "element_new_entry_book", CTRL + Key_B, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_book" ) ) ->setToolTip( i18n( "Add a book to the BibTeX file" ) );
     ( new KAction( i18n( "Booklet" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_booklet" ) ) ->setToolTip( i18n( "Add a booklet to the BibTeX file" ) );
     ( new KAction( i18n( "Electronic (IEEE)" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_electronic" ) ) ->setToolTip( i18n( "Add an electronic reference (IEEE) to the BibTeX file" ) );
     ( new KAction( i18n( "InBook" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_inbook" ) ) ->setToolTip( i18n( "Add a part of a book to the BibTeX file" ) );
     ( new KAction( i18n( "InCollection" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_incollection" ) ) ->setToolTip( i18n( "Add a publication which is part of some collection to the BibTeX file" ) );
-    ( new KAction( i18n( "InProceedings" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_inproceedings" ) ) ->setToolTip( i18n( "Add a publication which is part of some proceedings to the BibTeX file" ) );
+    ( new KAction( i18n( "InProceedings" ), "element_new_entry_inproceedings", CTRL + Key_I, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_inproceedings" ) ) ->setToolTip( i18n( "Add a publication which is part of some proceedings to the BibTeX file" ) );
     ( new KAction( i18n( "Manual" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_manual" ) ) ->setToolTip( i18n( "Add a manual to the BibTeX file" ) );
     ( new KAction( i18n( "MastersThesis" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_mastersthesis" ) ) ->setToolTip( i18n( "Add a master or diploma thesis to the BibTeX file" ) );
     ( new KAction( i18n( "Misc" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_misc" ) ) ->setToolTip( i18n( "Add a not specified document or reference to the BibTeX file" ) );
     ( new KAction( i18n( "PhDThesis" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_phdthesis" ) ) ->setToolTip( i18n( "Add a PhD thesis to the BibTeX file" ) );
-    ( new KAction( i18n( "Proceedings" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_proceedings" ) ) ->setToolTip( i18n( "Add some proceedings to the BibTeX file" ) );
+    ( new KAction( i18n( "Proceedings" ), "element_new_entry_phdthesis" , CTRL + Key_P, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_proceedings" ) ) ->setToolTip( i18n( "Add some proceedings to the BibTeX file" ) );
     ( new KAction( i18n( "TechReport" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_techreport" ) ) ->setToolTip( i18n( "Add a technical report to the BibTeX file" ) );
     ( new KAction( i18n( "Unpublished" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_unpublished" ) ) ->setToolTip( i18n( "Add an unpublished document to the BibTeX file" ) );
+    ( new KAction( i18n( "Preamble" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_preamble" ) ) ->setToolTip( i18n( "Add a preamble to the BibTeX file" ) );
     ( new KAction( i18n( "Comment" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_comment" ) ) ->setToolTip( i18n( "Add a comment to the BibTeX file" ) );
     ( new KAction( i18n( "Macro" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_macro" ) ) ->setToolTip( i18n( "Add a macro to the BibTeX file" ) );
 
@@ -391,6 +484,8 @@ void KBibTeXPart::setupActions( QWidget *parentWidget )
     m_actionShowMacros = new KToggleAction( i18n( "Show &Macros" ), 0, this, SLOT( slotToggleShowSpecialElements() ), actionCollection(), "view_showmacros" );
     m_actionShowMacros->setToolTip( i18n( "Show the macros of a BibTeX document" ) );
 
+    m_actionUseInPipe = new KToggleAction( i18n( "Accept input from other programs" ), 0, this , SLOT( slotUseInPipe() ), actionCollection(), "useinpipe" );
+
     connect( m_documentWidget, SIGNAL( listViewSelectionChanged( int ) ), this, SLOT( slotUpdateMenu( int ) ) );
 
     if ( m_mainWindow != NULL )
@@ -422,6 +517,8 @@ bool KBibTeXPart::slotNewElement()
             elementType = "comment";
         else if ( strcmp( sender() ->name(), "element_new_macro" ) == 0 )
             elementType = "macro";
+        else if ( strcmp( sender() ->name(), "element_new_preamble" ) == 0 )
+            elementType = "preamble";
         else
             return FALSE;
 
@@ -464,15 +561,18 @@ void KBibTeXPart::writeSettings()
 
 void KBibTeXPart::slotUpdateMenu( int numSelectedItems )
 {
+    m_documentWidget->updateViewDocumentMenu();
+    m_documentWidget->updateAssignKeywords();
+
     m_actionEditElement->setEnabled( numSelectedItems == 1 );
     m_actionDeleteElement->setEnabled( isReadWrite() && numSelectedItems > 0 );
     m_actionEditCut->setEnabled( isReadWrite() && numSelectedItems > 0 );
     m_actionEditCopy->setEnabled( numSelectedItems > 0 );
     m_actionEditCopyRef->setEnabled( numSelectedItems > 0 );
+    m_actionElementSendToLyX->setEnabled( numSelectedItems > 0 );
     m_actionMenuSearchWebsites->setEnabled( numSelectedItems == 1 );
     m_actionViewDocument->setEnabled( numSelectedItems == 1 && m_actionViewDocument->popupMenu() ->count() > 0 );
-
-    m_documentWidget->updateViewDocumentMenu( );
+    m_actionAssignKeywords->setEnabled( numSelectedItems == 1 );
 }
 
 void KBibTeXPart::slotUndoChanged( bool undoAvailable )
@@ -480,16 +580,56 @@ void KBibTeXPart::slotUndoChanged( bool undoAvailable )
     m_actionEditUndo->setEnabled( undoAvailable );
 }
 
+void KBibTeXPart::slotUseInPipe()
+{
+    if ( m_inPipe == NULL && QFile::exists( inPipeFilename ) )
+    {
+        KMessageBox::error( widget(), QString( i18n( "Some other KBibTeX instance is using the pipe.\nIf this assumption is wrong, please delete \"%1\"." ) ).arg( inPipeFilename ), i18n( "Pipe already in use" ) );
+    }
+    else
+    {
+        if ( m_inPipe != NULL )
+        {
+// remove pipe
+            m_inPipe->close();
+            m_inPipe->remove();
+            delete m_inPipe;
+            m_inPipe = NULL;
+            QFile::remove ( inPipeFilename );
+        }
+        else
+        {
+// create pipe
+            if ( ::mkfifo( inPipeFilename.latin1(), 0600 ) == 0 )
+            {
+                m_inPipe = new QFile( inPipeFilename );
+                if ( !m_inPipe->open( IO_ReadOnly ) || fcntl( m_inPipe->handle(), F_SETFL, O_NONBLOCK ) < 0 )
+                {
+                    m_inPipe->close();
+                    m_inPipe = NULL;
+                }
+                if ( m_inPipe == NULL )
+                    QFile::remove ( inPipeFilename );
+            }
+
+            if ( m_inPipe == NULL )
+                KMessageBox::error( widget(), QString( i18n( "Could not create pipe at \"%1\"." ) ).arg( inPipeFilename ), i18n( "Error creating pipe" ) );
+        }
+    }
+
+    m_actionUseInPipe->setChecked( m_inPipe != NULL );
+}
+
 void KBibTeXPart::slotSearchWebsites( int id )
 {
     KBibTeX::Settings * settings = KBibTeX::Settings::self();
-    m_documentWidget->searchWebsites( settings->searchURLs[ id - 1 ] ->url );
+    m_documentWidget->searchWebsites( settings->searchURLs[ id - 1 ] ->url, settings->searchURLs[ id - 1 ] ->includeAuthor );
 }
 
 void KBibTeXPart::slotSearchOnlineDatabases( int id )
 {
     for ( QValueList<KBibTeX::WebQuery*>::ConstIterator it = webQueryList.begin(); it != webQueryList.end(); ++it )
-        if ( ( *it ) ->uniqueId() == id )
+        if (( *it ) ->uniqueId() == id )
         {
             m_documentWidget->doWebQuery( *it );
             break;
@@ -518,11 +658,10 @@ void KBibTeXPart::slotDeferredInitialization()
 {
     if ( factory() != NULL )
     {
-        kdDebug() << "KBibTeXPart::slotDeferredInitialization BEGIN" << endl;
         m_documentWidget->setFactory( factory(), this );
+        m_documentWidget->setOnlineDatabaseSearchMenu( m_actionMenuSearchOnlineDatabases->popupMenu() );
         m_documentWidget->deferredInitialization();
         m_initializationDone = TRUE;
-        kdDebug() << "KBibTeXPart::slotDeferredInitialization END" << endl;
     }
     else
     {
@@ -580,7 +719,7 @@ KInstance* KBibTeXPartFactory::instance()
 {
     if ( !s_instance )
     {
-        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.1.5" );
+        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.2" );
         s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
         s_instance = new KInstance( s_about );
     }
