@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2004-2006 by Thomas Fischer                             *
+*   Copyright (C) 2004-2007 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -19,30 +19,23 @@
 ***************************************************************************/
 #include <qwidget.h>
 #include <qdom.h>
-#include <qlayout.h>
-#include <qlineedit.h>
-#include <qspinbox.h>
-#include <qlabel.h>
-#include <qpushbutton.h>
-#include <qcombobox.h>
 #include <qapplication.h>
-#include <qbuffer.h>
-#include <qhttp.h>
 #include <qstringlist.h>
+#include <qspinbox.h>
 #include <qcstring.h>
 #include <qregexp.h>
 
 #include <kdialog.h>
-#include <kurllabel.h>
-#include <kurl.h>
 #include <klocale.h>
+#include <klineedit.h>
+#include <kurl.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
-#include <kapplication.h>
-#include <kiconloader.h>
+#include <kio/netaccess.h>
 
 #include <entryfield.h>
 #include <value.h>
+#include <settings.h>
 #include "webquerypubmed.h"
 
 namespace KBibTeX
@@ -50,12 +43,12 @@ namespace KBibTeX
 
     WebQueryPubMed::WebQueryPubMed( QWidget* parent, const char* name ) : WebQuery( parent, name )
     {
-        wizard = new PubMedWizard( title(), parent, name );
+        wizard = new PubMed::PubMedWizard( title(), parent, name );
     }
 
     WebQueryPubMed::~WebQueryPubMed()
     {
-        // nothing
+        delete wizard;
     }
 
     BibTeX::File* WebQueryPubMed::query( )
@@ -66,246 +59,111 @@ namespace KBibTeX
             return NULL;
     }
 
-    QString WebQueryPubMed::title()
-    {
-        return i18n( "NCBI (PubMed)" );
-    }
-
     int WebQueryPubMed::uniqueId()
     {
         return 0x013ac4b8;
     }
 
-    PubMedWizard::PubMedWizard ( const QString &caption, QWidget* parent, const char* name ) : KDialogBase( parent, name, TRUE, "Import", Ok | Cancel, Ok, TRUE ), buffer( NULL ), bibtexFile( NULL )
+    QString WebQueryPubMed::title()
     {
-        setCaption( caption );
-        setupGUI();
-
-        http = new QHttp ( "eutils.ncbi.nlm.nih.gov" );
+        return i18n( "NCBI (PubMed)" );
     }
 
-    PubMedWizard::~PubMedWizard()
+    PubMed::PubMedWizard::PubMedWizard( const QString& caption, QWidget* parent, const char* name ) : WebQueryWizard( caption, i18n( "NCBI's Disclaimer and Copyright" ), "http://eutils.ncbi.nlm.nih.gov/About/disclaimer.html", FALSE, TRUE, parent, name )
     {
-        delete http;
+// nothing
     }
 
-    BibTeX::File *PubMedWizard::result()
+    PubMed::PubMedWizard::~PubMedWizard()
     {
-        return bibtexFile;
+// nothing
     }
 
-    void PubMedWizard::accept ()
-    {
-        bibtexFile = new BibTeX::File();
-        for ( QListViewItemIterator it( m_listViewResults, QListViewItemIterator::Selected ); it.current();++it )
-        {
-            ResultsListViewItem *item = dynamic_cast<ResultsListViewItem*>( *it );
-            if ( item != NULL )
-                bibtexFile->appendElement( BibTeX::File::cloneElement( item->entry() ) );
-        }
-
-        KDialogBase::accept();
-    }
-
-    int PubMedWizard::exec ()
-    {
-        buffer = new QBuffer( QByteArray( 0x1000000 ) );
-        int result = QDialog::exec();
-        delete buffer;
-
-        return result;
-    }
-
-    void PubMedWizard::startSearch()
+    void PubMed::PubMedWizard::startSearch()
     {
         setEnabled( FALSE );
         QApplication::setOverrideCursor( Qt::waitCursor );
         m_listViewResults->clear();
 
-        connect( http, SIGNAL( done( bool ) ), this, SLOT( databaseQueryDone( bool ) ) );
-        buffer->open( IO_ReadWrite );
-        KURL url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term=%1&retmax=%2&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( m_lineEditQuery->text() ).arg( m_spinBoxMaxHits->text() ) );
-        kdDebug() << "url: " << url.prettyURL() << endl;
-        QHttpRequestHeader header( "GET", url.prettyURL(), 1, 0 );
-        http->request( header, 0, buffer );
-    }
-
-    void PubMedWizard::databaseQueryDone( bool error )
-    {
-        disconnect( http, SIGNAL( done( bool ) ), this, SLOT( databaseQueryDone( bool ) ) );
-        buffer->close();
-
-        if ( error )
+        KURL url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term=%1&retmax=%2&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( m_lineEditQuery->text().replace( "%", "%25" ).replace( " ", "+" ).replace("?","%3F" ).replace("&","%26" ) ).arg( m_spinBoxMaxHits->text() ) );
+        QString tmpFile;
+        if ( KIO::NetAccess::download( url, tmpFile, NULL ) )
         {
-            setEnabled( TRUE );
-            QApplication::restoreOverrideCursor();
-            KMessageBox::error( this, QString( i18n( "Could not query NCBI's database:\n%1" ) ).arg( http->errorString() ) );
-            return ;
-        }
+            QFile inputFile( tmpFile );
+            QValueList<int> intList;
+            QXmlInputSource inputSource( &inputFile );
+            QXmlSimpleReader reader;
+            StructureParserQuery handler( &intList );
+            reader.setContentHandler( &handler );
+            reader.parse( &inputSource );
+            inputFile.close();
+            KIO::NetAccess::removeTempFile( tmpFile );
 
-        //         buffer->open( IO_ReadOnly );
-        //         QTextStream ts( buffer );
-        //         ts.setEncoding( QTextStream::UnicodeUTF8 );
-        //         QString dump = ts.read();
-        //         kdDebug() << "Fetched XML data BEGIN" << endl << dump << endl << "Fetched XML data END" << endl;
-        //         buffer->close();
-
-        QValueList<int> intList;
-        buffer->open( IO_ReadOnly );
-        QXmlInputSource input( buffer );
-        QXmlSimpleReader reader;
-        StructureParserQuery handler( &intList );
-        reader.setContentHandler( &handler );
-        reader.parse( &input );
-        buffer->close();
-
-        QString ids;
-        QValueList<int>::iterator it = intList.begin();
-        if ( it != intList.end() )
-        {
-            ids.append( QString::number( *it ) );
-            ++it;
-            for ( ; it != intList.end(); ++it )
+            QString ids;
+            QValueList<int>::iterator it = intList.begin();
+            if ( it != intList.end() )
             {
-                ids.append( "," );
                 ids.append( QString::number( *it ) );
+                ++it;
+                for ( ; it != intList.end(); ++it )
+                {
+                    ids.append( "," );
+                    ids.append( QString::number( *it ) );
+                }
             }
+
+            url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&retmode=xml&id=%1&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( ids ) );
+            if ( KIO::NetAccess::download( url, tmpFile, NULL ) )
+            {
+                QFile inputFile( tmpFile );
+                QDomDocument doc( "efetch'ed" );
+                doc.setContent( &inputFile );
+                QDomElement docElem = doc.documentElement();
+                ResultParser resultParser( m_listViewResults, doc.documentElement() );
+                inputFile.close();
+                KIO::NetAccess::removeTempFile( tmpFile );
+
+                setEnabled( TRUE );
+                QApplication::restoreOverrideCursor();
+                m_listViewResults->setFocus();
+            }
+            else
+            {
+                setEnabled( TRUE );
+                QApplication::restoreOverrideCursor();
+                KMessageBox::error( this, QString( i18n( "Could not query NCBI's database:\n%1" ) ).arg( KIO::NetAccess::lastErrorString() ) );
+                return ;
+            }
+
         }
-
-        KURL url = KURL( QString( "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&retmode=xml&id=%1&tool=KBibTeX&email=kbibtex@unix-ag.uni-kl.de" ).arg( ids ) );
-        kdDebug() << "url: " << url.prettyURL() << endl;
-
-        connect( http, SIGNAL( done( bool ) ), this, SLOT( fetchingSummaryDone( bool ) ) );
-        buffer->open( IO_ReadWrite );
-        QHttpRequestHeader header( "GET", url.prettyURL(), 1, 0 );
-        http->request( header, 0, buffer );
-
-    }
-
-    void PubMedWizard::fetchingSummaryDone( bool error )
-    {
-        disconnect( http, SIGNAL( done( bool ) ), this, SLOT( fetchingSummaryDone( bool ) ) );
-        buffer->close();
-
-        if ( error )
+        else
         {
             setEnabled( TRUE );
             QApplication::restoreOverrideCursor();
-            KMessageBox::error( this, QString( i18n( "Could not query NCBI's database:\n%1" ) ).arg( http->errorString() ) );
+            KMessageBox::error( this, QString( i18n( "Could not query NCBI's database:\n%1" ) ).arg( KIO::NetAccess::lastErrorString() ) );
             return ;
         }
-
-        //         buffer->open( IO_ReadOnly );
-        //         QTextStream ts( buffer );
-        //         ts.setEncoding( QTextStream::UnicodeUTF8 );
-        //         QString dump = ts.read();
-        //         kdDebug() << "Fetched XML data BEGIN" << endl << dump << endl << "Fetched XML data END" << endl;
-        //         buffer->close();
-
-        QDomDocument doc( "efetch'ed" );
-        doc.setContent( buffer );
-        QDomElement docElem = doc.documentElement();
-        ResultParser resultParser( m_listViewResults, doc.documentElement() );
-
-        setEnabled( TRUE );
-        QApplication::restoreOverrideCursor();
-        m_listViewResults->setFocus();
     }
 
-    void PubMedWizard::listViewResultsSelectionChanged( )
-    {
-        QListViewItemIterator it( m_listViewResults, QListViewItemIterator::Selected );
-
-        enableButtonOK( it.current() != NULL );
-    }
-
-    void PubMedWizard::setupGUI()
-    {
-        QWidget * mainWidget = new QWidget( this );
-        mainWidget->setMinimumSize( 640, 256 );
-        QGridLayout *gridLayout = new QGridLayout( mainWidget, 3, 5, 0, KDialog::spacingHint() );
-        gridLayout->setColStretch( 1, 10 );
-        gridLayout->setRowStretch( 1, 10 );
-        QLabel *label = new QLabel( i18n( "&Query string:" ), mainWidget );
-        gridLayout->addWidget ( label, 0, 0 );
-        m_lineEditQuery = new QLineEdit( mainWidget );
-        gridLayout->addWidget ( m_lineEditQuery, 0, 1 );
-        label->setBuddy( m_lineEditQuery );
-        label = new QLabel( i18n( "&Number of results:" ), mainWidget );
-        gridLayout->addWidget ( label, 0, 2 );
-        m_spinBoxMaxHits = new QSpinBox( 5, 5000, 1, mainWidget );
-        m_spinBoxMaxHits->setValue( 50 );
-        gridLayout->addWidget ( m_spinBoxMaxHits, 0, 3 );
-        label->setBuddy( m_spinBoxMaxHits );
-        m_pushButtonSearch = new QPushButton( i18n( "&Search" ), mainWidget );
-        gridLayout->addWidget ( m_pushButtonSearch, 0, 4 );
-        m_pushButtonSearch->setIconSet( QIconSet( SmallIcon( "find" ) ) );
-
-        m_listViewResults = new QListView( mainWidget );
-        m_listViewResults->addColumn( i18n( "Year" ), 64 );
-        m_listViewResults->addColumn( i18n( "Author" ), 128 );
-        m_listViewResults->addColumn( i18n( "Title" ), 512 );
-        m_listViewResults->setAllColumnsShowFocus( TRUE );
-        m_listViewResults->setSelectionMode( QListView::Extended );
-        gridLayout->addMultiCellWidget ( m_listViewResults, 1, 1, 0, 4 );
-
-        KURLLabel *disclaimerLabel = new KURLLabel( mainWidget );
-        disclaimerLabel->setText( i18n( "NCBI's Disclaimer and Copyright" ) );
-        disclaimerLabel->setURL( "http://eutils.ncbi.nlm.nih.gov/About/disclaimer.html" );
-        gridLayout->addMultiCellWidget ( disclaimerLabel, 2, 2, 0, 4 );
-
-        m_lineEditQuery->setFocus();
-        setMainWidget( mainWidget );
-        setButtonOK( KGuiItem( i18n( "&Import" ), "import", i18n( "Import selected items" ) ) );
-        enableButtonOK( FALSE );
-        setPlainCaption( i18n( "Import from NCBI (PubMed)" ) );
-
-        connect( disclaimerLabel, SIGNAL( leftClickedURL( const QString& ) ), kapp, SLOT( invokeBrowser( const QString& ) ) );
-        connect( m_listViewResults, SIGNAL( selectionChanged ( ) ), this, SLOT( listViewResultsSelectionChanged( ) ) );
-        connect( m_listViewResults, SIGNAL( clicked ( QListViewItem* ) ), this, SLOT( listViewResultsSelectionChanged( ) ) );
-        connect( m_pushButtonSearch, SIGNAL( clicked() ), this, SLOT( startSearch() ) );
-        connect( m_lineEditQuery, SIGNAL( returnPressed() ), this, SLOT( startSearch() ) );
-    }
-
-    StructureParserQuery::StructureParserQuery( QListView * listView ) : QXmlDefaultHandler(), m_listView( listView )
-    {
-        m_listView->clear();
-    }
-
-    StructureParserQuery::StructureParserQuery( QValueList<int> *intList ) : QXmlDefaultHandler(), m_listView( NULL ), m_intList( intList )
+    PubMed::StructureParserQuery::StructureParserQuery( QValueList<int> *intList ) : QXmlDefaultHandler(), m_intList( intList )
     {
         m_intList->clear();
     }
 
-    StructureParserQuery::~StructureParserQuery( )
+    PubMed::StructureParserQuery::~StructureParserQuery( )
     {
         // nothing
     }
 
-    bool StructureParserQuery::startElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & qName, const QXmlAttributes & /*atts*/ )
+    bool PubMed::StructureParserQuery::startElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & /*qName*/, const QXmlAttributes & /*atts*/ )
     {
         concatString = QString();
-
-        if ( qName == "ResultItem" && ( lastDbName == "books" || lastDbName == "pubmed" || lastDbName == "journals" ) )
-        {
-            bool ok;
-            int hits = lastCount.toInt( &ok );
-            if ( ok && hits > 0 )
-                new DatabasesListViewItem( m_listView, lastDbName, lastMenuName, hits );
-        }
-
         return TRUE;
     }
 
-    bool StructureParserQuery::endElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & qName )
+    bool PubMed::StructureParserQuery::endElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString & qName )
     {
-        if ( qName == "DbName" )
-            lastDbName = concatString;
-        else if ( qName == "MenuName" )
-            lastMenuName = concatString;
-        else if ( qName == "Count" )
-            lastCount = concatString;
-        else if ( qName == "Id" )
+        if ( qName == "Id" )
         {
             bool ok;
             int id = concatString.toInt( &ok );
@@ -316,13 +174,13 @@ namespace KBibTeX
         return TRUE;
     }
 
-    bool StructureParserQuery::characters ( const QString & ch )
+    bool PubMed::StructureParserQuery::characters( const QString & ch )
     {
         concatString.append( ch );
         return TRUE;
     }
 
-    ResultParser::ResultParser( QListView *listView, const QDomElement& rootElement )
+    PubMed::ResultParser::ResultParser( QListView *listView, const QDomElement& rootElement )
     {
         if ( rootElement.tagName() == "PubmedArticleSet" )
             for ( QDomNode n = rootElement.firstChild(); !n.isNull(); n = n.nextSibling() )
@@ -337,12 +195,12 @@ namespace KBibTeX
             }
     }
 
-    ResultParser::~ResultParser()
+    PubMed::ResultParser::~ResultParser()
     {
         // nothing
     }
 
-    void ResultParser::parsePubmedArticle( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parsePubmedArticle( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -352,7 +210,7 @@ namespace KBibTeX
         }
     }
 
-    void ResultParser::parseMedlineCitation( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parseMedlineCitation( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -376,9 +234,7 @@ namespace KBibTeX
                                 field = new BibTeX::EntryField( BibTeX::EntryField::ftJournal );
                                 entry->addField( field );
                             }
-                            BibTeX::Value *value = new BibTeX::Value ();;
-                            value->add( new BibTeX::ValueItem( e2.text(), FALSE ) );
-                            field->setValue( value );
+                            field->setValue( new BibTeX::Value( e2.text() ) );
                         }
                     }
                 }
@@ -387,10 +243,8 @@ namespace KBibTeX
 
     }
 
-    void ResultParser::parseArticle( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parseArticle( const QDomElement& element, BibTeX::Entry *entry )
     {
-        entry->setEntryType( BibTeX::Entry::etArticle );
-
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
             QDomElement e = n.toElement();
@@ -405,9 +259,7 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftTitle );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
             else if ( e.tagName() == "Pagination" )
             {
@@ -420,9 +272,7 @@ namespace KBibTeX
                         field = new BibTeX::EntryField( BibTeX::EntryField::ftPages );
                         entry->addField( field );
                     }
-                    BibTeX::Value *value = new BibTeX::Value ();;
-                    value->add( new BibTeX::ValueItem( medlinePgn.text(), FALSE ) );
-                    field->setValue( value );
+                    field->setValue( new BibTeX::Value( medlinePgn.text() ) );
                 }
             }
             // TF: I AM NOT SURE ABOUT COPYRIGHTS, SO FETCHING ABSTRACTS IS DISABLED FOR NOW
@@ -447,16 +297,14 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( "affiliation" );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
             else if ( e.tagName() == "AuthorList" )
                 parseAuthorList( e, entry );
         }
     }
 
-    void ResultParser::parseJournal( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parseJournal( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -470,9 +318,7 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftISSN );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
             else if ( e.tagName() == "JournalIssue" )
                 parseJournalIssue( e, entry );
@@ -484,14 +330,12 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftJournal );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
         }
     }
 
-    void ResultParser::parseJournalIssue( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parseJournalIssue( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -505,9 +349,7 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftVolume );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
             else if ( e.tagName() == "Issue" )
             {
@@ -517,16 +359,14 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftNumber );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), FALSE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
             else if ( e.tagName() == "PubDate" )
                 parsePubDate( e, entry );
         }
     }
 
-    void ResultParser::parsePubDate( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parsePubDate( const QDomElement& element, BibTeX::Entry *entry )
     {
         for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
         {
@@ -540,9 +380,7 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftYear );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( e.text(), TRUE ) );
-                field->setValue( value );
+                field->setValue( new BibTeX::Value( e.text() ) );
             }
             else if ( e.tagName() == "Month" )
             {
@@ -553,8 +391,8 @@ namespace KBibTeX
                     field = new BibTeX::EntryField( BibTeX::EntryField::ftMonth );
                     entry->addField( field );
                 }
-                BibTeX::Value *value = new BibTeX::Value ();;
-                value->add( new BibTeX::ValueItem( month, TRUE ) );
+                BibTeX::Value *value = new BibTeX::Value();
+                value->items.append( new BibTeX::MacroKey( month ) );
                 field->setValue( value );
             }
             else if ( e.tagName() == "MedlineDate" )
@@ -572,8 +410,8 @@ namespace KBibTeX
                             field = new BibTeX::EntryField( BibTeX::EntryField::ftYear );
                             entry->addField( field );
                         }
-                        BibTeX::Value *value = new BibTeX::Value ();;
-                        value->add( new BibTeX::ValueItem( QString::number( num ), TRUE ) );
+                        BibTeX::Value *value = new BibTeX::Value();
+                        value->items.append( new BibTeX::MacroKey( QString::number( num ) ) );
                         field->setValue( value );
                     }
                     else if ( !ok && ( *it ).length() == 3 )
@@ -584,8 +422,8 @@ namespace KBibTeX
                             field = new BibTeX::EntryField( BibTeX::EntryField::ftMonth );
                             entry->addField( field );
                         }
-                        BibTeX::Value *value = new BibTeX::Value ();;
-                        value->add( new BibTeX::ValueItem( ( *it ).lower(), TRUE ) );
+                        BibTeX::Value *value = new BibTeX::Value();
+                        value->items.append( new BibTeX::MacroKey(( *it ).lower() ) );
                         field->setValue( value );
                     }
                 }
@@ -593,7 +431,7 @@ namespace KBibTeX
         }
     }
 
-    void ResultParser::parseAuthorList( const QDomElement& element, BibTeX::Entry *entry )
+    void PubMed::ResultParser::parseAuthorList( const QDomElement& element, BibTeX::Entry *entry )
     {
         if ( element.attribute( "CompleteYN", "Y" ) == "Y" )
         {
@@ -625,56 +463,15 @@ namespace KBibTeX
                 field = new BibTeX::EntryField( BibTeX::EntryField::ftAuthor );
                 entry->addField( field );
             }
-            BibTeX::ValuePersons *value = new BibTeX::ValuePersons ();;
-            value->add( new BibTeX::ValueItem( authorList.join( " and " ), FALSE ) );
+            BibTeX::Value *value = new BibTeX::Value();
+            Settings *settings = Settings::self();
+            BibTeX::PersonContainer *personContainer = new BibTeX::PersonContainer( settings->editing_FirstNameFirst );
+            value->items.append( personContainer );
+            for ( QStringList::Iterator sli = authorList.begin(); sli != authorList.end(); ++sli )
+                personContainer->persons.append( new BibTeX::Person( *sli, settings->editing_FirstNameFirst ) );
             field->setValue( value );
         }
     }
 
-    DatabasesListViewItem::DatabasesListViewItem ( QListView * parent, const QString & dbName, const QString & dbTitle, int numHits ) : QListViewItem( parent ), m_dbName( dbName )
-    {
-        setText( 0, QString::number( numHits ) );
-        setText( 1, dbTitle );
-    }
-
-    QString DatabasesListViewItem::dbName()
-    {
-        return m_dbName;
-    }
-
-    ResultsListViewItem::ResultsListViewItem ( QListView * parent, BibTeX::Entry * entry ) : QListViewItem( parent ), m_entry( entry )
-    {
-        BibTeX::EntryField * field = entry->getField( BibTeX::EntryField::ftTitle );
-        if ( field != NULL && field->value() != NULL )
-            setText( 2, field ->value() ->plainString() );
-        field = entry->getField( BibTeX::EntryField::ftAuthor );
-        if ( field != NULL && field->value() != NULL )
-        {
-            BibTeX::ValuePersons * persons = dynamic_cast<BibTeX::ValuePersons*>( field->value() );
-            if ( persons != NULL )
-            {
-                QValueList<BibTeX::Person*> list = persons->persons();
-                QStringList authors;
-                for ( QValueList<BibTeX::Person*>::ConstIterator it = list.begin();it != list.end(); ++it )
-                    authors.append( ( *it ) ->text() );
-                setText( 1, authors.join( " and " ) );
-            }
-            else setText( 1, field ->value() ->plainString() );
-        }
-        field = entry->getField( BibTeX::EntryField::ftYear );
-        if ( field != NULL && field->value() != NULL )
-            setText( 0, field ->value() ->plainString() );
-    }
-
-    ResultsListViewItem::~ResultsListViewItem()
-    {
-        if ( m_entry != NULL )
-            delete m_entry;
-    }
-
-    BibTeX::Entry* ResultsListViewItem::entry()
-    {
-        return m_entry;
-    }
 }
 
